﻿from keras.models import model_from_json
import numpy as np
import time
import re


# 31-mer coding
def coding(mers):
    # amino acid dictionaries
    dic = {'A':0, 'C':1, 'D':2, 'E':3, 'F':4, 'G':5, 'H':6, 'I':7, 'K':8,
           'L':9, 'M':10, 'N':11, 'P':12, 'Q':13, 'R':14, 'S':15, 'T':16,
           'V':17, 'W':18, 'Y':19, 'Z':20}
    
    # coding
    vectors = []
    for mer in mers:
        coded_mer = [dic.get(aa) for aa in mer]
        vectors += [coded_mer]
    
    return vectors


# predicting
def predictor(protease, data, res_path):
    print("Loading model...")
    s3 = time.time()
    # load architecture
    with open('%s.json' % protease, 'r') as dd:
        model = dd.read()
    loaded_model = model_from_json(model)
    # load parameters
    loaded_model.load_weights('%s.h5' % protease)
    e3 = time.time()
    print("Time cost of loading model is %s seconds." % (e3 - s3))
    
    # predition
    print("Predicting! Please wait...")
    s4 = time.time()
    # extract all the 31-mers
    mers = []
    for line in data:
        seqs = re.split('[\t,]', line[1])[1:]
        mers += [mer for mer in seqs if len(mer) > 1]  # mer != '' or '*'
    print("There are %s candidate cleavage sites." % len(mers))
    
    # predicting probabilities of all the sites
    x_mers = np.array(coding(mers))
    mers_pred = loaded_model.predict(x_mers)
    
    # calculating the peptide digestibilities
    print("Calculating peptide detectabilities.")
    dig_ind = 0 # index of 31-mer
    results = []
    for line in data:
        pro_id, seqs = line
        seqs_list = seqs.split('\t')
        # The model predicted the probabilities of missed digestion!!!
        if len(seqs_list) == 4:
            pep, left_mer, right_mer, missed_mers = seqs_list
            # digestibility of the left 31-mer
            if left_mer != '*':
                left_dig = 1 - float(mers_pred[dig_ind])
                dig_ind += 1
            else:
                left_dig = float(1)
            # digestibility of the right 31-mer
            if right_mer != '*':
                right_dig = 1 - float(mers_pred[dig_ind])
                dig_ind += 1
            else:
                right_dig = float(1)
            # digestibilities of the missed 31-mers
            missed_sites = missed_mers.split(',')
            missed_probs = ''
            missed_dig = float(1)
            for site in missed_sites:
                if len(site) == 31:
                    prob = float(mers_pred[dig_ind])
                    dig_ind += 1
                    missed_probs += str(1 - prob) + ','
                    missed_dig *= prob
                elif site != '':
                    print(line)
                    print(site)
            # peptide digestibility
            dig_pro = left_dig * right_dig * missed_dig
            results += [[pro_id, pep, left_dig, right_dig, missed_probs]]
        elif len(seqs_list) == 1:
            results += [line + [''] * 3]
    e4 = time.time()
    print("Time cost of prediction is %s seconds." % (e4 - s4))
    
    # save results
    s5 = time.time()
    np.savetxt(res_path, results, fmt='%s\t%s\t%s\t%s\t%s',
               delimiter='\t', newline='\n',
               header = 'Protein id\tPeptide sequence\t'\
                           'Digestibility of the N-terminal site\t'\
                            'Digestibility of the C-terminal site\t'\
                                'Digestibility of the missed site(s)',
               comments='')
    e5 = time.time()
    print("Time cost of saving results is %s seconds." % (e5 - s5))

